import audioop
import sys
from datetime import datetime
import threading

from sound_localizer import SoundLocalizer

class ChannelConnector:
    def __init__(self, sample_rate, sample_width=2):
        self.cached_audios = {}
        self.previous_time = None
        self.lock = threading.Lock()
        self.timer = None
        self.sample_width = sample_width
        self.sound_localizer = SoundLocalizer(sample_rate)

    def add_audio(self, audio_bytes, channel):
        with self.lock:
            self.cached_audios[channel] = audio_bytes

            if self.previous_time is None:
                self.previous_time = datetime.now()
                # Schedule the reset and comparison after 1 second
                self.timer = threading.Timer(1.0, self.reset_and_compare)
                self.timer.start()

    def reset_and_compare(self):
        with self.lock:
            if 0 in self.cached_audios and 1 in self.cached_audios:
                self.compare(self.cached_audios[0], self.cached_audios[1])
            self.reset()

    def reset(self):
        self.previous_time = None
        self.cached_audios = {}
        if self.timer:
            self.timer.cancel()
            self.timer = None

    def compare(self, audio_left, audio_right):
        left_rms = audioop.rms(audio_left, self.sample_width)
        right_rms = audioop.rms(audio_right, self.sample_width)

        if left_rms > 30 or right_rms > 30:
            print(f"Left channel RMS: {left_rms}", file=sys.stderr)
            print(f"Right channel RMS: {right_rms}", file=sys.stderr)
            print('left' if left_rms > right_rms else 'right', file=sys.stderr)

            angle = self.sound_localizer.estimate_source_angle(audio_left, audio_right)
            print(f'Approximate angle: {angle}', file=sys.stderr)
