import numpy as np
from scipy.signal import correlate
import math
import sys

class SoundLocalizer:
    def __init__(self, sample_rate, channels=2):
        self.sample_rate = sample_rate
        self.channels = channels

    def _compute_tdoa(self, sig1, sig2, framerate):
        correlation = correlate(sig1, sig2, mode='full')
        lag = np.argmax(correlation) - (len(sig1) - 1)
        tdoa = lag / framerate
        return tdoa

    def _calculate_angle(self, tdoa, speed_of_sound, distance_between_mics, mic_angles):
        angles = []
        normalized_tdoa = tdoa * speed_of_sound / distance_between_mics
        normalized_tdoa = np.clip(normalized_tdoa, -1, 1)
        tdoa_angle = math.asin(normalized_tdoa) * 180 / math.pi
        for angle in mic_angles:
            angles.append(angle + tdoa_angle)
        return np.mean(angles)

    def estimate_source_angle(self, audio_left, audio_right, \
    speed_of_sound=343.0, distance_between_mics=0.1):

        audio_left = np.frombuffer(audio_left, dtype=np.int16)
        audio_left = audio_left.reshape(-1, self.channels)
        audio_right = np.frombuffer(audio_right, dtype=np.int16)
        audio_right = audio_right.reshape(-1, self.channels)

        mic1 = audio_left[:, 0]
        mic2 = audio_left[:, 1]
        mic3 = audio_right[:, 0]
        mic4 = audio_right[:, 1]

        tdoa_12 = self._compute_tdoa(mic1, mic2, self.sample_rate)
        tdoa_13 = self._compute_tdoa(mic1, mic3, self.sample_rate)
        tdoa_14 = self._compute_tdoa(mic1, mic4, self.sample_rate)
        print(tdoa_12, tdoa_13, tdoa_14, file=sys.stderr)

        mic_angles = [-33, -11, 11, 33]

        angle_12 = self._calculate_angle(tdoa_12, speed_of_sound, distance_between_mics, mic_angles[:2])
        angle_13 = self._calculate_angle(tdoa_13, speed_of_sound, distance_between_mics, mic_angles[:3])
        angle_14 = self._calculate_angle(tdoa_14, speed_of_sound, distance_between_mics, mic_angles)

        final_angle = np.mean([angle_12, angle_13, angle_14])

        return final_angle
