from wav_composer import WavComposer
import datetime

class WavRecorder:
    def __init__(self, output_directory):
        self._header = b''
        self._data_chunks = []
        self._output_directory = output_directory
        self._is_recording = False

    def start_recording(self):
        self._is_recording = True

    def set_header(self, header):
        self._header = header

    def append_data(self, data):
        if not self._is_recording:
            raise RuntimeError("Cannot append data. Recorder is not recording.")
        self._data_chunks.append(data)

    def save_recording(self, buf=None):
        self._write_to_file(buf)
        self._reset_recorder()

    def _write_to_file(self, buf=None):
        wav_data = self._header + b''.join(data for data in self._data_chunks)
        output_path = buf if buf else f'{self._output_directory}/{filename}'
        WavComposer.compose(output_path, wav_data)

        print(f'Saved recording to {output_path}')

    def _reset_recorder(self):
        self._header = b''
        self._data_chunks.clear()
        self._is_recording = False

    def is_recording(self):
        return self._is_recording

